﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace ArchiveViewer.ViewModel
{
    public class MainViewModel : ViewModelBase, IDisposable
    {
        #region Properties

        string _arcPath;
        SpiMngx.FileInfoCollection _items = null;

        /// <summary>
        /// プラグイン管理
        /// </summary>
        SpiMngx.SpiManager _manager = new SpiMngx.SpiManager();

        /// <summary>
        /// ファイル選択ダイアログ用のファイルフィルタ
        /// </summary>
        public string FileFilter
        {
            get { return _manager.GetFileFilter(); }
        }

        /// <summary>ファイル情報</summary>
        public ObservableCollection<FileInfoViewModel> Items
        {
            get { return _Items; }
            set
            {
                if (_Items != value)
                {
                    _Items = value;
                    OnPropertyChanged("Items");
                }
            }
        }
        private ObservableCollection<FileInfoViewModel> _Items = new ObservableCollection<FileInfoViewModel>();

        #endregion

        /// <summary>
        /// リソースの解放
        /// </summary>
        public void Dispose()
        {
            _manager.Dispose();
            if (_items != null)
            {
                _items.Dispose();
            }
        }

        /// <summary>
        /// 書庫ファイルを開く
        /// </summary>
        /// <param name="path"></param>
        public bool OpenArchive(string path)
        {
            _arcPath = path;

            // 対応プラグインを取得
            var plugin = _manager.QueryArchivePlugin(_arcPath);
            if (plugin == null)
                return false;

            // 書庫情報を取得
            if (_items != null)
            {
                _items.Dispose();
            }
            _items = plugin.GetArchiveInfo(path);

            // 書庫内のファイルを列挙
            this.Items.Clear();
            foreach (var item in _items)
            {
                this.Items.Add(new FileInfoViewModel(item));
            }
            return true;
        }

        /// <summary>
        /// 書庫内のファイルの保存
        /// </summary>
        /// <param name="destPath">出力フォルダ</param>
        /// <param name="items">ファイル情報</param>
        public bool SaveFiles(string destPath, System.Collections.IList items)
        {
            // 対応プラグインを取得
            var plugin = _manager.QueryArchivePlugin(_arcPath);
            if (plugin == null)
            {
                return false;
            }

            // ファイルを保存
            foreach (FileInfoViewModel info in items)
            {
                plugin.GetFile(_arcPath, info.FileInfo,
                    System.IO.Path.Combine(destPath, info.FileName));
            }
            return true;
        }

        /// <summary>
        /// ファイルから画像を読み出す
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public System.Windows.Media.Imaging.BitmapSource GetImage(FileInfoViewModel item)
        {
            // 対応プラグインを取得
            var plugin = _manager.QueryArchivePlugin(_arcPath);
            if (plugin == null)
            {
                return null;
            }

            // ファイルを展開
            using (var buffer = plugin.GetFile(_arcPath, item.FileInfo))
            {
                // 画像ファイルを読み込み
                var imagePlugin = _manager.QueryImagePlugin(item.FileName, buffer);
                if (imagePlugin == null)
                {
                    return null;
                }
                return imagePlugin.GetPicture(buffer);
            }
        }

        /// <summary>
        /// ファイルから画像を読み出す
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public System.Windows.Media.Imaging.BitmapSource GetImage(string path)
        {
            // 対応プラグインを取得
            var imagePlugin = _manager.QueryImagePlugin(path);
            if (imagePlugin == null)
            {
                return null;
            }
            return imagePlugin.GetPicture(path);
        }
    }
}
