﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ArchiveViewer
{
    /// <summary>
    /// アーカイブ操作と画像表示のサンプル
    /// </summary>
    /// <remarks>
    /// あまりこてこてのMVVMにならないようにしました。
    /// プラグインの操作はViewModel.MainViewModelにまとめています。
    /// </remarks>
    public partial class Window1 : Window
    {
        ViewModel.MainViewModel _viewModel = new ViewModel.MainViewModel();

        public Window1()
        {
            InitializeComponent();

            this.DataContext = _viewModel;
            this.Closing += (o, x) => _viewModel.Dispose();
        }

        /// <summary>
        /// 画像を表示
        /// </summary>
        /// <param name="path"></param>
        /// <param name="image"></param>
        void ShowImage(string path, BitmapSource image)
        {
            var view = new ImageView(path, image);
            view.Owner = this;
            view.Show();
        }

        /// <summary>
        /// ファイルを開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OpenFileCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            // ファイル選択ダイアログ
            var dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.Filter = _viewModel.FileFilter;
            if (dlg.ShowDialog() ?? false)
            {
                // 書庫ファイルなら開く
                if (_viewModel.OpenArchive(dlg.FileName))
                {
                    this.Title = "ArchiveViewer - " + System.IO.Path.GetFileName(dlg.FileName);
                }
                else
                {
                    // 画像なら表示
                    var image = _viewModel.GetImage(dlg.FileName);
                    if (image != null)
                    {
                        ShowImage(dlg.FileName, image);
                    }
                }
            }
        }

        /// <summary>
        /// 選択ファイルの保存
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveFileCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if (_listView.SelectedItems.Count != 0)
            {
                // 保存先の選択
                var dlg = new System.Windows.Forms.FolderBrowserDialog();
                dlg.Description = "Select folder";
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    // ファイルを保存
                    _viewModel.SaveFiles(dlg.SelectedPath, _listView.SelectedItems);
                }
            }
        }

        /// <summary>
        /// 画像ファイルを開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OpenImageCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if (_listView.SelectedItems.Count != 0)
            {
                var item = _listView.SelectedItems[0] as ViewModel.FileInfoViewModel;
                if (item != null)
                {
                    // ファイルを展開し、画像なら表示
                    var image = _viewModel.GetImage(item);
                    if (image != null)
                    {
                        ShowImage(item.FileName, image);
                    }
                }
            }
        }
    }
}
