﻿Imports System.Collections.ObjectModel

Public Class MainViewModel
    Inherits ViewModelBase
    Implements IDisposable

#Region "Properties"
    Dim _arcPath As String
    Dim _items As SpiMngx.FileInfoCollection
    Dim _manager As New SpiMngx.SpiManager()

    ''' <summary>
    ''' ファイル選択ダイアログ用のファイルフィルタ
    ''' </summary>
    Public ReadOnly Property FileFilter() As String
        Get
            Return _manager.GetFileFilter()
        End Get
    End Property

    ''' <summary>
    ''' ファイル情報
    ''' </summary>
    Private _itemViews As New ObservableCollection(Of FileInfoViewModel)
    Public Property Items() As ObservableCollection(Of FileInfoViewModel)
        Get
            Return _itemViews
        End Get
        Set(ByVal value As ObservableCollection(Of FileInfoViewModel))
            _itemViews = value
        End Set
    End Property

#End Region

#Region " IDisposable Support "
    ' このコードは、破棄可能なパターンを正しく実装できるように Visual Basic によって追加されました。
    Public Sub Dispose() Implements IDisposable.Dispose
        _manager.Dispose()
        If _items IsNot Nothing Then
            _items.Dispose()
        End If
    End Sub
#End Region

    ''' <summary>
    ''' 書庫ファイルを開く
    ''' </summary>
    ''' <param name="path"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function OpenArchive(ByVal path As String) As Boolean
        _arcPath = path

        ' 対応プラグインを取得
        Dim plugin As SpiMngx.ArchivePlugin = _manager.QueryArchivePlugin(_arcPath)
        If plugin Is Nothing Then Return False

        ' 書庫情報を取得
        If _items IsNot Nothing Then _items.Dispose()
        _items = plugin.GetArchiveInfo(_arcPath)

        ' 書庫内のファイルを列挙
        Me.Items.Clear()
        For Each item As SpiMngx.FileInfo In _items
            Me.Items.Add(New FileInfoViewModel(item))
        Next
        Return True
    End Function

    ''' <summary>
    ''' 書庫内のファイルの保存
    ''' </summary>
    ''' <param name="destPath">出力フォルダ</param>
    ''' <param name="items">ファイル情報</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function SaveFiles(ByVal destPath As String, ByVal items As IList) As Boolean
        ' 対応プラグインを取得
        Dim plugin As SpiMngx.ArchivePlugin = _manager.QueryArchivePlugin(_arcPath)
        If plugin Is Nothing Then Return False

        ' ファイルを保存
        For Each info In items
            plugin.GetFile(_arcPath, info.FileInfo, _
                           System.IO.Path.Combine(destPath, info.FileName))
        Next

        Return True
    End Function

    ''' <summary>
    ''' ファイルから画像を読み出す
    ''' </summary>
    ''' <param name="item"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetImage(ByVal item As FileInfoViewModel) As BitmapSource
        ' 対応プラグインを取得
        Dim plugin As SpiMngx.ArchivePlugin = _manager.QueryArchivePlugin(_arcPath)
        If plugin Is Nothing Then Return Nothing

        ' ファイルを展開
        Using buffer As SpiMngx.LocalStream = plugin.GetFile(_arcPath, item.FileInfo)
            Dim imagePlugin As SpiMngx.ImagePlugin = _manager.QueryImagePlugin(item.FileName, buffer)
            If imagePlugin Is Nothing Then Return Nothing
            Return imagePlugin.GetPicture(buffer)
        End Using
    End Function

    ''' <summary>
    ''' ファイルから画像を読み出す
    ''' </summary>
    ''' <param name="path"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetImage(ByVal path As String) As BitmapSource
        ' 対応プラグインを取得
        Dim imagePlugin As SpiMngx.ImagePlugin = _manager.QueryImagePlugin(path)
        If imagePlugin Is Nothing Then Return Nothing
        Return imagePlugin.GetPicture(path)
    End Function

End Class
