﻿Class Window1 
    Dim _viewModel As New MainViewModel()

    Public Sub New()
        InitializeComponent()

        Me.DataContext = _viewModel
        AddHandler Me.Closing, AddressOf Me.Window_Closing
    End Sub

    Private Sub Window_Closing(ByVal sender As System.Object, ByVal e As System.ComponentModel.CancelEventArgs)
        _viewModel.Dispose()
    End Sub

    ''' <summary>
    ''' 画像を表示
    ''' </summary>
    ''' <param name="path"></param>
    ''' <param name="image"></param>
    ''' <remarks></remarks>
    Sub ShowImage(ByVal path As String, ByVal image As BitmapSource)
        Dim view As New ImageView(path, image)
        view.Owner = Me
        view.Show()
    End Sub

    ''' <summary>
    ''' ファイルを開く
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OpenFileCommand_Executed(ByVal sender As System.Object, ByVal e As System.Windows.Input.ExecutedRoutedEventArgs)

        ' ファイル選択ダイアログ
        Dim dlg As New Microsoft.Win32.OpenFileDialog()
        dlg.Filter = _viewModel.FileFilter
        If dlg.ShowDialog() = True Then

            ' 書庫ファイルなら開く
            If _viewModel.OpenArchive(dlg.FileName) Then
                Me.Title = "ArchiveViewer - " + System.IO.Path.GetFileName(dlg.FileName)
            Else
                ' 画像なら表示
                Dim image = _viewModel.GetImage(dlg.FileName)
                If image IsNot Nothing Then ShowImage(dlg.FileName, image)
            End If
        End If
    End Sub

    ''' <summary>
    ''' 選択ファイルの保存
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SaveFileCommand_Executed(ByVal sender As System.Object, ByVal e As System.Windows.Input.ExecutedRoutedEventArgs)

        If _listView.SelectedItems.Count <> 0 Then

            ' 保存先の選択
            Dim dlg As New System.Windows.Forms.FolderBrowserDialog()
            dlg.Description = "Select folder"
            If dlg.ShowDialog() = System.Windows.Forms.DialogResult.OK Then

                ' ファイルを保存
                _viewModel.SaveFiles(dlg.SelectedPath, _listView.SelectedItems)
            End If
        End If
    End Sub

    ''' <summary>
    ''' 画像ファイルを開く
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OpenImageCommand_Executed(ByVal sender As System.Object, ByVal e As System.Windows.Input.ExecutedRoutedEventArgs)

        If _listView.SelectedItems.Count <> 0 Then
            Dim item = TryCast(_listView.SelectedItems(0), FileInfoViewModel)
            If item IsNot Nothing Then

                ' ファイルを展開し、画像なら表示
                Dim image = _viewModel.GetImage(item)
                If image IsNot Nothing Then ShowImage(item.FileName, image)
            End If
        End If
    End Sub

End Class
